# jgtfxc.py

import os
import json
import datetime
import pandas as pd
from forexconnect import ForexConnect, fxcorepy

#@STCGoal Future use
#import forexconnect as fxcon
# fxcon.ForexConnect.create_order_request
# fxcon.ForexConnect.create_request
# fxcon.ForexConnect.get_table
# fxcon.ForexConnect.get_timeframe
# fxcon.ForexConnect.get_timeframe

# from . import common_samples as jgtfxcommon
from . import jgtfxcommon


fx=None
quotes_count=None
stayConnected=False

def login_forexconnect(user_id, password, url, connection):
    fx = ForexConnect()
    try:
        fx.login(user_id, password, url, connection, "", "", jgtfxcommon.session_status_changed)
    except Exception as e:
        jgtfxcommon.print_exception(e)
    return fx

#@STCIssue Matching our original connect
def connect(quiet=True):
    global fx,quotes_count
    if fx is not None:
        if not quiet:
            print("Already connected")
        return
    
    config=readconfig()

    str_user_id = config['user_id']
    str_password = config['password']
    str_url = config['url']
    str_connection = config['connection']
    quotes_count = config['quotes_count']

    fx = login_forexconnect(str_user_id, str_password, str_url, str_connection)


def logout_forexconnect(fx):
    try:
        fx.logout()
    except Exception as e:
        jgtfxcommon.print_exception(e)
def disconnect(quiet=False):
    global fx
    if fx is None:
        print("Not connected")
        return
    logout_forexconnect(fx)
    fx=None

def status():
    global fx
    if fx is None:
        print("STATUS : Not connected")
        return
    print("STATUS : Connected")


def readconfig():
    # Try reading config file from current directory
    config_file = 'config.json'
    if not os.path.isfile(config_file):
        # If config file not found, check home directory
        home_dir = os.path.expanduser("~")
        config_file = os.path.join(home_dir, 'config.json')

    # Read config file
    with open(config_file, 'r') as file:
        config = json.load(file)
        return config


def get_price_history(instrument, timeframe, datefrom=None, dateto=None,quotes_count_spec=None,quiet=True):
    global quotes_count,fx

    if quotes_count_spec is None:
        quotes_count_spec=quotes_count

    connect()
    if dateto is None:
        dateto = datetime.datetime.now()    

    try:
        if not quiet:
            print("")
            print("Requesting a price history...")
        history = fx.get_history(instrument, timeframe, datefrom, dateto, quotes_count_spec)

        current_unit, _ = ForexConnect.parse_timeframe(timeframe)

        if current_unit == fxcorepy.O2GTimeFrameUnit.TICK:
            data = pd.DataFrame(history, columns=['Date', 'Bid', 'Ask'])
        else:
            data = pd.DataFrame(history, columns=['Date','BidOpen','BidHigh','BidLow','BidClose','AskOpen','AskHigh','AskLow','AskClose','Volume'])

        return data

    finally:
        if not stayConnected:
            disconnect()
        else:
            print("---we stay connected---")
        #logout_forexconnect(fx)


def get_price_history1(instrument, timeframe, datefrom=None, dateto=None):
    # Try reading config file from current directory
    config=readconfig()

    str_user_id = config['user_id']
    str_password = config['password']
    str_url = config['url']
    str_connection = config['connection']
    quotes_count = config['quotes_count']

    if dateto is None:
        dateto = datetime.datetime.now()

    fx = login_forexconnect(str_user_id, str_password, str_url, str_connection)

    try:
        print("")
        print("Requesting a price history...")
        history = fx.get_history(instrument, timeframe, datefrom, dateto, quotes_count)

        current_unit, _ = ForexConnect.parse_timeframe(timeframe)

        if current_unit == fxcorepy.O2GTimeFrameUnit.TICK:
            data = pd.DataFrame(history, columns=['Date', 'Bid', 'Ask'])
        else:
            data = pd.DataFrame(history, columns=['Date', 'BidOpen', 'BidHigh', 'BidLow', 'BidClose', 'Volume'])

        return data

    finally:
        logout_forexconnect(fx)


def get_price_history_printed(instrument, timeframe, datefrom=None, dateto=None):
    data = get_price_history(instrument, timeframe, datefrom, dateto)

    if data is not None:
        if 'Ask' in data.columns:
            print("Date, Bid, Ask")
        else:
            print("Date, BidOpen, BidHigh, BidLow, BidClose, Volume")

        for index, row in data.iterrows():
            values = row.values.tolist()
            print(",".join(str(value) for value in values))


# Example usage
#get_price_history_printed(instrument='EUR/USD', timeframe='m1')

#fx
def getAccount():
    account = fx.getAccount()
    print(account)
    return account

def getSubscribedSymbols():
    symbols = fx.getSubscribedSymbols()
    print(symbols)
    return symbols


import datetime as dt
import pandas as pd
import os

from . import JGTPDHelper as jpd

#import jgtpy.JGTFXCMWrapper as jfx
from . import jgtfxc as jfx

from . import JGTConfig as jgtcnf


renameColumns=True
addOhlc=True
stayConnected=False

def stayConnectedSetter(_v):
  global stayConnected
  stayConnected=_v
  jfx.stayConnected=_v
  jfx.connect()
  
cleanseOriginalColumns=True
useLocal=False
con=None


def mk_fn(_instrument,_timeframe,_ext):
  """Make a file name with instrument and timeframe

  Args:
      _instrument (str): symbol
      _timeframe (str): TF
      _ext (str): ext name "csv"

  Returns:
      str: file name
  """
  _tf = _timeframe
  _i= _instrument.replace('/','-')
  if _timeframe == 'm1':
    _tf = _timeframe.replace('m1','mi1')
  _fn= _i + '_' + _tf + '.' + _ext
  return _fn.replace('..','.')

def mk_fullpath(_instrument,_timeframe,_ext,_path):
  _fn=mk_fn(_instrument,_timeframe,_ext)
  _r= _path + '/'+_fn
  return _r.replace('..','.').replace('//','/')





def pds_add_ohlc_stc_columns(dfsrc):
  if not 'Open' in dfsrc.columns:
    dfsrc['Open'] = dfsrc[['BidOpen', 'AskOpen']].mean(axis=1)
    dfsrc['High'] = dfsrc[['BidHigh', 'AskHigh']].mean(axis=1)
    dfsrc['Low'] = dfsrc[['BidLow', 'AskLow']].mean(axis=1)
    dfsrc['Close'] = dfsrc[['BidClose', 'AskClose']].mean(axis=1)
    #Median
    dfsrc['Median']= ((dfsrc['High'] + dfsrc['Low']) / 2)
    return dfsrc


def __cleanse_original_columns(dfsrc,_quiet=True):
  dfsrc=jpd.pds_cleanse_original_columns(dfsrc,_quiet)
  return dfsrc


def getSubscribed():
  return jfx.con.get_instruments_for_candles()

def connect(quiet=True):  
  return jfx.connect(quiet)

def disconnect(quiet=True):
  return jfx.disconnect(quiet)

def tryConnect():
  try:
    con=connect()
  except ConnectionError:
    print("Connection error")

def status():
  return jfx.status()

def getPH_from_local1(instrument,timeframe):
  srcpath=jgtcnf.get_pov_local_data_filename(instrument,timeframe)
  df=pd.read_csv(srcpath,compression=jgtcnf.local_fn_compression,index_col='Date')
  return df

def getPH_from_local(instrument,timeframe,quiet=True):
  # Define the file path based on the environment variable or local path
  data_path = os.environ.get('JGTPY_DATA', './data')
  fn=mk_fn(instrument,timeframe,'csv')
  srcpath=mk_fullpath(instrument,timeframe,'csv',data_path)
  if not quiet  :
    print(srcpath)
  # df=pd.read_csv(srcpath,index_col='Date')
  df=pd.read_csv(srcpath)
  return df

def getPH_to_file(instrument,timeframe,quote_count=335,start=None,end=None,with_index=True,quiet=True,compressed=False):
  # Define the file path based on the environment variable or local path
  data_path = os.environ.get('JGTPY_DATA', './data')
  fpath=mk_fullpath(instrument,timeframe,'csv',data_path)
  df=getPH(instrument,timeframe,quote_count,start,end,False,quiet)
  if compressed:
    df.to_csv(fpath,compression=jgtcnf.local_fn_compression)
  else:
    df.to_csv(fpath)
  return fpath

def getPH2file(instrument,timeframe,quote_count=335,start=None,end=None,with_index=True,quiet=True,compressed=False):
  return getPH_to_file(instrument,timeframe,quote_count,start,end,with_index,quiet,compressed)

def getPH(instrument,timeframe,quote_count=335,start=None,end=None,with_index=True,quiet=True):
  """Get Price History from Broker

  Args:
      instrument (str): symbal
      timeframe (str): TF
      quote_count (int, optional): nb bar to retrieve. Defaults to 335.
      start (str, optional): start DateTime. Defaults to None.
      end (str, optional): end DateTime range. Defaults to None.
      with_index (bool, optional): Return DataFrame with Index. Defaults to True.
      quiet  (bool, optional): stay calm ;)

  Returns:
      pandas.DataFrame: DF with price histories
  """
  df = pd.DataFrame()
  if not useLocal:
    con=connect(quiet=quiet)

    p=jfx.get_price_history(instrument, timeframe, start,end, quote_count+89)
    #print(p)
    df=pd.DataFrame(p,columns=['Date','BidOpen','BidHigh','BidLow','BidClose','AskOpen','AskHigh','AskLow','AskClose','Volume'])
    #print("------------DATAFRAME----------------")
    #print(df)

    #mask = (df['Date'] > start) & (df['Date'] <= end)
    #df = df.loc[mask]

    if not stayConnected:
      con=disconnect(quiet=quiet)
    if renameColumns:
      df=df.rename(columns={'bidopen': 'BidOpen', 'bidhigh': 'BidHigh','bidclose':'BidClose','bidlow':'BidLow','askopen': 'AskOpen', 'askhigh': 'AskHigh','askclose':'AskClose','asklow':'AskLow','tickqty':'Volume','date':'Date'})
      df= df.astype({'Volume':int})
    if with_index:
      df.index.rename('Date',inplace=True)
  else:
    #Read from local
    df =getPH_from_local(instrument,timeframe)
    if with_index:
      df.index.rename('Date',inplace=True)
    if start != None:
      mask = (df['Date'] > end) & (df['Date'] <= start)
      df = df.loc[mask]
  
  # df=df.rename_axis(index=None, columns=None)
  # df.index.rename('Date', inplace=True)
  if addOhlc and renameColumns:
    df=pds_add_ohlc_stc_columns(df)
  if cleanseOriginalColumns:
    df=__cleanse_original_columns(df)
  # Set 'Date' column as the index
  df.set_index('Date', inplace=True)
  return df


def getPHByRange(instrument,timeframe,start=None,end=None,with_index=True,quiet=True):
  """Get Price History from Broker

  Args:
      instrument (str): symbal
      timeframe (str): TF
      start (str, optional): start DateTime. Defaults to None.
      end (str, optional): end DateTime range. Defaults to None.
      with_index (bool, optional): Return DataFrame with Index. Defaults to True.
      quiet  (bool, optional): stay calm ;)

  Returns:
      pandas.DataFrame: DF with price histories
  """
  df = pd.DataFrame()
  #if disconnected:
  # con=connect()
  if not useLocal:
    con=connect(quiet=quiet)
    df=jfx.con.get_candles(instrument, period=timeframe,with_index=with_index,start=start,end=end)
    if not stayConnected:
      con=disconnect(quiet=quiet)
    if renameColumns:
      df=df.rename(columns={'bidopen': 'BidOpen', 'bidhigh': 'BidHigh','bidclose':'BidClose','bidlow':'BidLow','askopen': 'AskOpen', 'askhigh': 'AskHigh','askclose':'AskClose','asklow':'AskLow','tickqty':'Volume','date':'Date'})
      df= df.astype({'Volume':int})
      if with_index:
        df.index.rename('Date',inplace=True)
  else:
    #Read from local
    df =getPH_from_local(instrument,timeframe) 
    if with_index:
        df.index.rename('Date',inplace=True)
    mask = (df['Date'] > end) & (df['Date'] <= start)
    df = df.loc[mask]
  #print(df)
  #df.to_csv("data.csv")
  #mask = (df['Date'] > end) & (df['Date'] <= start)
  #df = df.loc[mask]
  # df=df.rename_axis(index=None, columns=None)
  # df.index.rename('Date', inplace=True)
  if addOhlc and renameColumns:
    df=pds_add_ohlc_stc_columns(df)
  if cleanseOriginalColumns:
    df=__cleanse_original_columns(df)
  return df



def getPresentBarAsList(_df):
  _paf =_df.iloc[-1:]
  _pa = _paf.to_dict(orient='list')
  _dtctx=str(_paf.index.values[0])
  _pa['Date'] = _dtctx
  return _pa

def getLastCompletedBarAsList(_df):
  _paf =_df.iloc[-2:-1]
  _pa = _paf.to_dict(orient='list')
  _dtctx=str(_paf.index.values[0])
  _pa['Date'] = _dtctx
  return _pa

  



#@title PD Columen Cleanup Functions

def jgtpd_drop_col_by_name(_df,colname,_axis = 1,_quiet=False):
  """Drop Column in DF by Name

  Args:
        _df (DataFrame source)
        ctxcolname (column name from)
        _axis (  axis)
        _quiet (quiet output)
        
  Returns:
    Clean DataFrame 
  """
  if colname in _df.columns:
    return _df.drop(_df.loc[:, colname:colname].columns,axis = _axis)
  else:
    if not _quiet:
      print('Col:' + colname + ' was not there')
    return _df

def __ids_cleanse_ao_peak_secondary_columns(_df,_quiet=False):
  _df=jgtpd_drop_col_by_name(_df,'p0',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p1',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p2',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p3',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p4',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p5',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p6',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p7',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p8',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p9',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p10',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p11',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p12',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p13',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p14',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p15',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p16',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p17',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p18',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p19',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p20',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p21',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p22',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p23',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p24',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p25',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p26',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p27',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p28',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p29',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'p30',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n0',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n1',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n2',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n3',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n4',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n5',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n6',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n7',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n8',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n9',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n10',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n11',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n12',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n13',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n14',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n15',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n16',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n17',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n18',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n19',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n20',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n21',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n22',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n23',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n24',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n25',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n26',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n27',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n28',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n29',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'n30',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao0',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao1',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao2',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao3',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao4',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao5',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao6',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao7',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao8',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao9',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao10',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao11',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao12',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao13',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao14',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao15',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao16',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao17',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao18',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao19',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao20',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao21',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao22',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao23',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao24',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao25',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao26',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao27',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao28',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao29',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pao30',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao0',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao1',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao2',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao3',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao4',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao5',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao6',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao7',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao8',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao9',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao10',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao11',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao12',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao13',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao14',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao15',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao16',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao17',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao18',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao19',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao20',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao21',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao22',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao23',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao24',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao25',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao26',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao27',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao28',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao29',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nao30',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pac0',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pac',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pac1',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pac2',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pac3',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pac4',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pac5',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pac6',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pac7',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pac8',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'pac9',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nac',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nac0',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nac1',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nac2',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nac3',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nac4',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nac5',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nac6',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nac7',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nac8',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'nac9',1,_quiet) 
  return _df

def pds_cleanse_original_columns(_df,_quiet=True):
  _df=jgtpd_drop_col_by_name(_df,'AskHigh',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'BidHigh',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'AskLow',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'BidLow',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'AskClose',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'BidClose',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'BidOpen',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'AskOpen',1,_quiet)
  return _df

def pds_cleanse_extra_columns(_df,_quiet=True):
  _df=pds_cleanse_original_columns(_df,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'LowisBellowJaw',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'HighisAboveJaw',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'LowisBellowTeeth',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'HighisAboveTeeth',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'HighisAboveLips',1,_quiet)
  _df=jgtpd_drop_col_by_name(_df,'LowisBellowLips',1,_quiet)
  _df=__ids_cleanse_ao_peak_secondary_columns(_df,_quiet)
  if not _quiet:
    print("Columns cleanup was executed")
  return _df